package event

import (
	"fmt"
	"github.com/disgoorg/disgo/discord"
	"github.com/disgoorg/disgo/events"
	"grow.rievo.dev/discordBots/cmd/welcomebot/config"
	"log"
	"time"
)

func JoinEvent(event *events.GuildMemberJoin) {
	// on join send message in welcome
	user := event.Member.User
	if event.GuildID.String() != config.RegisterGuildID.String() {
		log.Printf("ERROR: join from an other guild")
		return
	}
	guild, err := event.Client().Rest().GetGuild(config.RegisterGuildID, false)
	if err != nil {
		log.Printf("ERRPR: could not find guild: %v", err)
		return
	}

	_, err = event.Client().Rest().CreateMessage(config.ChannelWelcomeID,
		discord.NewMessageCreateBuilder().
			SetContent(user.Mention()).
			SetEphemeral(false).
			AddActionRow(discord.NewSecondaryButton("approve", "approve")).
			SetEmbeds(discord.NewEmbedBuilder().
				SetColor(16068661).
				SetTimestamp(time.Now()).
				//SetThumbnail(user.EffectiveAvatarURL(discord.WithSize(1024))).
				SetThumbnail(fmt.Sprintf("https://app11.rvo.one/api/v1/discord/avatar/%s", user.ID.String())).
				AddField(
					fmt.Sprintf("Welcome to the %v Server", guild.Name),
					fmt.Sprintf("Welcome %v (`%v`) on %v 🍎", user.Mention(), user.Username, guild.Name), false).
				AddField("Next Step", fmt.Sprintf("schau dir die %v und spaeter die %v an", config.ChannelRulesMention, config.ChannelInfoMention), false).
				Build()).
			Build(),
	)
	if err != nil {
		log.Printf("ERROR: on sending join message: %v", err)
		return
	}
}

func LeaveEvent(event *events.GuildMemberLeave) {
	user := event.User

	if event.GuildID.String() != config.RegisterGuildID.String() {
		log.Printf("ERROR: leave from an other guild")
		return
	}
	guild, err := event.Client().Rest().GetGuild(config.RegisterGuildID, false)
	if err != nil {
		log.Printf("ERROR: could not find guild: %v", err)
		return
	}

	_, err = event.Client().Rest().CreateMessage(config.ChannelWelcomeID,
		discord.NewMessageCreateBuilder().
			SetContent(
				fmt.Sprintf("%v (`%v`) left %v", user.Mention(), user.Username, guild.Name)).
			Build(),
	)
	if err != nil {
		log.Printf("ERROR: on sending leave message: %v", err)
		return
	}
}

func ReactionEvent(event *events.ComponentInteractionCreate) {

	if event.Channel().ID() != config.ChannelWelcomeID {
		// reaction is not in welcome chat
		return
	}

	approverID := event.User().ID
	if approverID.String() == event.Client().ID().String() {
		// reaction user is bot itself
		return
	}
	if event.ButtonInteractionData().CustomID() != "approve" {
		// not approve button
		return
	}
	approver, err := event.Client().Rest().GetMember(config.RegisterGuildID, approverID)
	if err != nil {
		log.Printf("ERROR: could not find user: %v", err)
		return
	}
	hasPermission := func() bool {
		for _, roleID := range approver.RoleIDs {
			if roleID.String() == config.RoleAppleCoreID.String() {
				return true
			}
		}
		return false
	}()

	if hasPermission == false {
		err = event.CreateMessage(discord.NewMessageCreateBuilder().
			SetEphemeral(true).
			SetContent("you don't have permissions to do this").
			Build())
		if err != nil {
			log.Printf("ERROR: could not respond to reaction: %v", err)
			return
		}
		return
	}

	messageID := event.Message.ID
	message, err := event.Client().Rest().GetMessage(config.ChannelWelcomeID, messageID)
	if err != nil {
		log.Printf("ERROR: could not find message: %v", err)
		return
	}
	if len(message.Mentions) != 1 {
		// there should be only one mention in the message
		return
	}
	user := message.Mentions[0]

	// add role to user (bot role needs to be above roleAppleID)
	err = event.Client().Rest().AddMemberRole(config.RegisterGuildID, user.ID, config.RoleAppleID)
	if err != nil {
		log.Printf("ERROR: not able to add role: %v", err)
		return
	}

	// update embed (and time)
	if len(message.Embeds) != 1 {
		// there should be only one embed in the message
		return
	}
	embed := message.Embeds[0]
	updatedMessage := discord.NewEmbedBuilder().
		SetColor(16068661).
		SetTimestamp(time.Now()).
		SetThumbnail(embed.Thumbnail.URL).
		AddFields(embed.Fields...).
		AddField("Approval", fmt.Sprintf("%v approved by %v (`%v`)", user.Mention(), approver.Mention(), approver.User.Username), false).
		Build()

	err = event.UpdateMessage(discord.NewMessageUpdateBuilder().
		SetContent(message.Content).
		SetEmbeds(updatedMessage).
		ClearContainerComponents().
		Build())
	if err != nil {
		log.Printf("ERROR: could not update message: %v", err)
		return
	}
}
